
CHARSET = 'utf-8'

def to_str(s, errors='strict'):
    """
    Theoretically http://www.python.org/dev/peps/pep-0263/ for this and 
    # -*- coding: utf-8 -*-
    should help. But it don't .
    """
    
    if not isinstance(s, basestring):
        try:
            return str(s)
        except UnicodeEncodeError:
            return unicode(s).encode(CHARSET, errors)
    elif isinstance(s, unicode):
        return s.encode(CHARSET, errors)
    else:
        return s
    



#    spotsh - The SpotCloud command-line utility.
#    Copyright (C) 2011  Enomaly
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from urlparse import urljoin, urlparse, urlsplit
import urllib2
import os.path
import socket
from email.Generator import _make_boundary

try:
    from json import loads, dumps
except:
    from simplejson import loads, dumps
    
def upload_package(client, pkg_path):
    
    upload_url = get_upload_url(client)
    result = upload(client, upload_url, pkg_path)
    return result

def get_upload_url(client):

    upload_url = client.get('/api/package/upload').read()
    return upload_url

def upload(client, upload_url, pkg_path):
    
    content_type, prefix, suffix = build_headers(os.path.basename(pkg_path))
    body_length = os.path.getsize(pkg_path)
    content_length = len(prefix) + body_length + len(suffix)

    url_parts = urlsplit(upload_url)    
    
    client.connection.putrequest('POST', url_parts[2])
    client.connection.putheader('Content-Type', content_type)
    client.connection.putheader('Content-Length', content_length)
    client.connection.endheaders()

    client.connection.send(prefix)
    
    body_file = open(pkg_path)
    try:
        blocksize=1024
        data=body_file.read(blocksize)
        while data:
            print '\rUploading: %2d%%...' % ((body_file.tell()*100)/body_length),
            sys.stdout.flush() 
            client.connection.sock.sendall(data)
            data=body_file.read(blocksize)
    except socket.error, v:
        if v.args[0] == 32:
            client.connection.close()
        raise
    print "\rUpload completed.        "

    client.connection.send(suffix)
    response = client.connection.getresponse() 
    
    if response.status == 200:
        return loads(response.read())
    if response.status in (302, 303):            
        location = response.getheader('Location')
        return loads(urllib2.urlopen(location).read())
        
    raise Exception('Unknown return status: %s %s' % (response.status, response.reason))

def build_headers(filename, key_name='file'):
    
    boundary = '----------ThIs_Is_tHe_bouNdaRY_$'
    CRLF = '\r\n'
    PREFIX = []
    PREFIX.append('--' + boundary)
    PREFIX.append('Content-Disposition: form-data; name="%s"; filename="%s"' % (key_name, filename))
    PREFIX.append('Content-Type: %s' % 'application/octet-stream')
    PREFIX.append('')
    PREFIX.append('')
    
    SUFFIX = ['']
    SUFFIX.append('--' + boundary + '--')
    SUFFIX.append('')
    
    content_type = 'multipart/form-data; boundary=%s' % boundary
    
    return content_type, CRLF.join(PREFIX), CRLF.join(SUFFIX)




#    spotsh - The SpotCloud command-line utility.
#    Copyright (C) 2011  Enomaly
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import re
import string
import tempfile
import tarfile
import subprocess
import shutil
import sys
from cStringIO import StringIO
from subprocess import call
from gzip import GzipFile

try:
    from hashlib import md5
except ImportError:
    from md5 import new as md5

try:
    import xml.etree.ElementTree as ET # in python >=2.5
except ImportError:
    try:
        import cElementTree as ET # effbot's C module
    except ImportError:
        try:
            import elementtree.ElementTree as ET # effbot's pure Python module
        except ImportError:
            try:
                import lxml.etree as ET # ElementTree API using libxml2
            except ImportError:
                import warnings
                warnings.warn("could not import ElementTree "
                              "(http://effbot.org/zone/element-index.htm)")
                raise
try:
    import uuid
    def gen_uuid():
        return str(uuid.uuid1())
except ImportError:
    import subprocess
    def gen_uuid():
        p = subprocess.Popen('uuidgen', stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        data, _ = p.communicate()        
        if p.wait() != 0:
            raise ImportError('Please install python uuid module (http://pypi.python.org/pypi/uuid/) or uuidgen utility')
        return data.strip()

DISK_TYPE_HD = 'hd'
DISK_TYPE_CDROM = 'cdrom'

MB = 1024*1024

class BaseDisk(object):
    type = None
    image_type = None
    internal_name = None
    def __str__(self):
        return str(self.__unicode__())
    
class ExistingImageDisk(BaseDisk):
    def __init__(self, type, path):
        self.type = type
        self.path = path
        
        self.size = os.path.getsize(path)
        self.image_type = os.path.splitext(path)[1][1:]
        
        self.gzipped_path = None
        
    def get_gzipped_path(self):
        if not self.gzipped_path:
            self.gzipped_path = gzip_file(self.path)
        return self.gzipped_path        
    
    def get_hash(self):
        return calculate_hash(self.path)
    
    def get_size(self):
        return self.size
    
    def cleanup(self):
        if self.gzipped_path:
            os.unlink(self.gzipped_path)
            self.gzipped_path = None

    def __unicode__(self):
        return '%s: %s (%d MB)' % (self.type, self.path, self.size / MB)
    
def build_package(name, path):
    
    cfg = PackageConfig(name,
                        [ExistingImageDisk(DISK_TYPE_HD, path)],
                        DISK_TYPE_HD,
                        True,
                        True,
                        True,
                        True,
                        True)
    
    for idx, disk in enumerate(cfg.disks):
        disk.internal_name = '%s%d.%s.gz' % (disk.type, idx, disk.image_type)

    tf_name = '%s.xvm2' % cfg.name
    tf = tarfile.open(tf_name, 'w')
    
    try:    
    
        try:

            m1 = StringIO(build_manifest1(cfg))
            m1_tf = tarfile.TarInfo(name='description.xml')
            m1_tf.size = len(m1.getvalue())
            tf.addfile(m1_tf, fileobj=m1)
            
            m2 = StringIO(build_manifest2(cfg))
            m2_tf = tarfile.TarInfo(name='manifest.txt')
            m2_tf.size = len(m2.getvalue())
            tf.addfile(m2_tf, fileobj=m2)
                
            for disk in cfg.disks:
                tf.add(disk.get_gzipped_path(), disk.internal_name)

        finally:
            
            tf.close()      

    except KeyboardInterrupt, AttributeError:
        os.unlink(tf_name)
                
    except:
        os.unlink(tf_name)
        raise
        
    for disk in cfg.disks:
        try:
            disk.cleanup()
        except Exception, e:
            sys.exit(1, 'Failed to cleanup disk "%s"' % disk)     
                
    return tf_name


def build_manifest1(cfg):
    m = ET.Element('domain')
    ET.SubElement(m, 'name').text=cfg.name
    ET.SubElement(m, 'uuid').text=gen_uuid()
    
    os = ET.SubElement(m, 'os')
    ET.SubElement(os, 'type').text='hvm'
    ET.SubElement(os, 'boot', dev=cfg.boot_from)
    
    features = ET.SubElement(m, 'features')
    if cfg.use_pae:
        ET.SubElement(features, 'pae')
    if cfg.use_acpi:
        ET.SubElement(features, 'acpi')
    if cfg.use_apic:
        ET.SubElement(features, 'apic')
    
    devices = ET.SubElement(m, 'devices')
    
    for idx, d in enumerate(cfg.disks):        
        device = 'disk'
        if d.type == DISK_TYPE_CDROM:
            device = 'cdrom'
        disk = ET.SubElement(devices, 'disk', device=device, type='file')
        devname = 'hd%s' % string.ascii_lowercase[idx]
        ET.SubElement(disk, 'source', file=d.internal_name)
        ET.SubElement(disk, 'target', dev=devname)
    
    return ET.tostring(m)

def build_manifest2(cfg):
    m = ''
    for disk in cfg.disks:
        name = disk.internal_name
        if name.endswith('.gz'):
            name = name[:-3]
        m += '%s %d %s\n' % (name, disk.get_size(), disk.get_hash())
    return m

def gzip_file_(file_path):
    "original gzip_file()"
    gz_path = tempfile.mkstemp(suffix='.gz')[1]
    try:
        proc = subprocess.Popen("nice gzip --fast -c %s > %s " % (file_path, gz_path),
                                shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        _, stderr = proc.communicate()
        if proc.wait() != 0:
            raise Exception(stderr.strip())
        return gz_path
    except KeyboardInterrupt:
        os.unlink(gz_path)
        # TODO: and return None without exception ?
    except:
        os.unlink(gz_path)
        raise

def gzip_file(file_path):
    """pro: pure python, display progress
    cons: 20% slower compare to gzip -1"""
    bs = 10 * MB
    f = open(file_path)
    gz_path = tempfile.mkstemp(suffix='.gz')[1]
    gf = GzipFile(gz_path, "wb", 1)

    try:
        data = f.read(bs)
        chunk = 1
        chunks = int(round(os.path.getsize(file_path) / bs + 0.5))
        while len(data):
            print '\rCompressing: %2d%%...' % ((chunk*100)/chunks),
            sys.stdout.flush() 
            gf.write(data)
            data = f.read(bs)
            chunk += 1
        gf.close()
    except:
        os.unlink(gz_path)
        raise    
        
    return gz_path

def create_empty_disk(size):
    gz_path = tempfile.mkstemp(suffix='.gz')[1]
    bs=1024*1024
    count=size/bs
    call("dd if=/dev/zero bs=%d count=%d 2> /dev/null | nice gzip --fast > %s" % (bs, count, gz_path), shell=True)
    return gz_path

def calculate_empty_hash(size):
    
    """Calculate and return the hash for an empty disk, using the
    provided size."""
    
    empty_hash = md5()
    block_size = MB
    zero       = '\0' * block_size
    
    while size > block_size:
        
        empty_hash.update(zero)
        size -= block_size
        
    empty_hash.update(zero[:size])
    
    return empty_hash.hexdigest()

def calculate_hash(file_path):
    
    """Calculate and return the hash for the provided file path."""
    
    file_hash  = md5()
    block_size = MB
    
    with open(file_path, 'rb') as ifile:

        for buf in ifile.read(block_size):
            
            file_hash.update(buf)
            
    return file_hash.hexdigest()
            
class PackageConfig(object):
    def __init__(self, name, disks, boot_from, use_pae, use_acpi, use_apic, use_virtio, use_virtblk):
        self.name = name
        self.disks = disks
        self.boot_from = boot_from
        self.use_pae = use_pae
        self.use_acpi = use_acpi
        self.use_apic = use_apic
        self.use_virtio = use_virtio
        self.use_virtblk = use_virtblk
        
    def __repr__(self):
        return 'PackageConfig(%r, %r, %r, %r, %r, %r, %r, %r)' % (self.name, self.disks, self.boot_from, self.use_pae, self.use_acpi, self.use_apic, self.use_virtio, self.use_virtblk)





#    spotsh - The SpotCloud command-line utility.
#    Copyright (C) 2011  Enomaly
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


from utils import to_str


class Instance(object):
    
    def __init__(self, instance):
        
        self.instance = instance
        
    def __repr__(self):
        
        return '\n'\
               'ID:     %s\n'\
               'Key:    %s\n'\
               'State:  %s\n'\
               'IP:     %s\n'\
               % (self.instance['id'],\
                  self.instance['key'],\
                  self.instance['state'],\
                  self.instance['ip'])        
                 
    def __str__(self):
        
        return self.__repr__()

class Appliance(object):
    
    def __init__(self, appliance):
        
        self.appliance = appliance
        
    def __repr__(self):
        
        return '\n'\
               'Name:   %s\n'\
               'Size:   %s\n'\
               'Key:    %s\n'\
               % (self.appliance['name'],\
                  self.appliance['size'],\
                  self.appliance['key'])        
                 
    def __str__(self):
        
        return self.__repr__()
        
                 
class Provider(object):
    
    def __init__(self, provider):

        self.provider = provider
        
    def __repr__(self):
        
        return to_str(
               '\n'\
               'ID:          %s\n'\
               'Continent:   %s\n'\
               'Country:     %s\n'\
               'City:        %s\n'\
               'Min Cost:    %s\n'\
               'Max Cost:    %s\n'\
               % (self.provider['id'],
                  self.provider['continent'],
                  self.provider['country'],
                  self.provider['city'],
                  self.provider['min_cost'],
                  self.provider['max_cost'])
        )
                 
    def __str__(self):
        return self.__repr__()

class Hardware(object):
    
    def __init__(self, hardware):
        
        self.hardware = hardware
        
    def __repr__(self):
        
        return '\n'\
               'Name:        %s\n'\
               'Cost:        %s\n'\
               'CPU:         %s\n'\
               'Memory:      %s\n'\
               'Max Hours:   %s\n'\
               'Key:         %s\n'\
               % (self.hardware['name'],
                  self.hardware['cost'],
                  self.hardware['cpu'],
                  self.hardware['memory'],
                  self.hardware['max_hours'],
                  self.hardware['key'])        
                 
    def __str__(self):
        
        return self.__repr__()




#    spotsh - The SpotCloud command-line utility.
#    Copyright (C) 2011  Enomaly
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import os.path
import sys
import optparse
import pickle
import httplib

try:
    from json import loads, dumps
except:
    from simplejson import loads, dumps
from oauth import oauth

import model
from api import SpotCloud
from packagebuilder import build_package, PackageConfig, DISK_TYPE_HD,\
                           ExistingImageDisk
from packageuploader import upload_package


ACCESS_TOKEN_PATH = os.path.expanduser('~/.spotcloud_api_token')


class BaseCommand(object):
    
    option_list = [
        optparse.make_option('-z',
                             '--host',
                             default='spotcloud.appspot.com',
                             dest='_host',
                             help='Host')
    ]
    
    def __init__(self, options):
        self.option_list.extend(options)
        self.parser = optparse.OptionParser(usage=self.usage(),
                                            version=self.get_version(),
                                            option_list=self.option_list)
        self.options, self.args = self.parser.parse_args()
        try:
            self.client = self.makeClient()
        except RuntimeError:
            print self.usage()
        
    def makeClient(self):

        loaded = False
        access_token = None
        
        try:
            access_token = pickle.load(open(ACCESS_TOKEN_PATH))
            loaded = True
        except:
            pass
        
        consumer = oauth.OAuthConsumer('anonymous', 'anonymous')

        client = SpotCloud(self.options._host, consumer, access_token=access_token)
        
        if not client.access_token:
            client.fetch_oauth_request_token()
            authorization_url = client.get_oauth_authorize_url()
            print 'Please confirm token on page %s and press Enter' % authorization_url
            try:
                raw_input()
            except KeyboardInterrupt:
                sys.exit(0)
            try:
                client.fetch_oauth_access_token()
            except KeyError:
                sys.exit('Unauthorized')
            access_token = client.access_token
                   
        if not loaded:
            pickle.dump(client.access_token, open(ACCESS_TOKEN_PATH, 'w'))        
            
        return client
        
        
    def check_response(self, response, expected_status=httplib.OK):
        if response.status != expected_status:
            body = response.read()
            if body:
                body = "body: %s" % body
            sys.exit("""status: %s reason: "%s" %s""" % (response.status,
                                                response.reason, body))
        
        
    def parse_response(self, response, expected_status=httplib.OK):
        self.check_response(response, expected_status)
        
        if expected_status != httplib.NO_CONTENT:
            try:
                body = response.read()
                return loads(body)
            except ValueError:
                sys.exit('API error.  Please try again later. Response body: %s' % body)

        # TODO: and return None by default ?
        
        
    def get_version(self):

        return '0.0.1'


class MainCommand(object):
      
    def __init__(self):
           
        commands = {'instance': InstanceCommand,
                    'appliance': ApplianceCommand,
                    'provider': ProviderCommand,
                    'hardware': HardwareCommand,
                    'logout': LogoutCommand,}
        
        if len(sys.argv) >=2 and sys.argv[1] in commands.keys():
            commands[sys.argv[1]]()
            
        else:
            print self.usage()
        
    def usage(self):
        
        return '\nspotsh instance \n'\
               '       appliance \n'\
               '       provider\n'\
               '       hardware\n'\
               '       logout\n'\
        
        
class InstanceCommand(BaseCommand):
    
    command_options = (
        optparse.make_option('-l',
                             '--list',
                             action='store_true',
                             dest='_list',
                             help='List all instances'),
                    
        optparse.make_option('-k',
                             '--key',
                             default=None,
                             dest='_key',
                             help='Retrieve instance by key'),
                    
        optparse.make_option('-d',
                             '--delete',
                             default=None,
                             dest='_delete',
                             help='Delete instance by key'),
                    
        optparse.make_option('-c',
                             '--create',
                             action='store_true',
                             dest='_create',
                             help='Create a new instance using a provider, appliance, and hardware key'),
                             
        optparse.make_option('-p',
                             '--provider',
                             default=None,
                             dest='_provider',
                             help='Provider key used when creating an instance'),
                             
        optparse.make_option('-a',
                             '--appliance',
                             default=None,
                             dest='_appliance',
                             help='Appliance key used when creating an instance'),
                             
        optparse.make_option('-m',
                             '--hardware',
                             default=None,
                             dest='_hardware',
                             help='Hardware key used when creating an instance')
    )
    
    def __init__(self):
        
        super(InstanceCommand, self).__init__(self.command_options)
              
        if self.options._list:
            response = self.client.get('/api/v1/buyer/instances/list')
            instances = self.parse_response(response)
            for instance in instances:
                print model.Instance(instance)
            
        elif self.options._key:
            response = self.client.get('/api/v1/buyer/instance/%s' % self.options._key)
            instance = self.parse_response(response)
            print model.Instance(instance)

        elif self.options._delete:
            response = self.client.delete('/api/v1/buyer/instance/%s' % self.options._delete)
            self.parse_response(response, expected_status=httplib.NO_CONTENT)
            print 'Deleted\n'  
        
        elif self.options._create:
            response = self.client.post('/api/v1/buyer/instances/list',
                                        data = dict(provider=self.options._provider,
                                                    appliance=self.options._appliance,
                                                    hardware=self.options._hardware))
            instance = self.parse_response(response, expected_status=httplib.CREATED)
            print model.Instance(instance)
        
        else:
            self.parser.print_help()
            
    def usage(self):
        return 'spotsh instance \n'
   
   
class ApplianceCommand(BaseCommand):
    
    command_options = (
        optparse.make_option('-l',
                             '--list',
                             action='store_true',
                             dest='_list',
                             help='List all appliances'),
                    
        optparse.make_option('-k',
                             '--key',
                             default=None,
                             dest='_key',
                             help='Retrieve appliance by key'),
                    
        optparse.make_option('-d',
                             '--delete',
                             default=None,
                             dest='_delete',
                             help='Delete appliance by key'),
                    
        optparse.make_option('-b',
                             '--build',
                             default=None,
                             dest='_build',
                             help='Build a SpotCloud appliance from a raw disk'),
                             
        optparse.make_option('-n',
                             '--name',
                             default='New Appliance',
                             dest='_name',
                             help='Name used when building a new appliance'),
                             
        optparse.make_option('-u',
                             '--upload',
                             default=None,
                             dest='_upload',
                             help='Upload an XVM2 appliance')
    )
    
    def __init__(self):
        
        super(ApplianceCommand, self).__init__(self.command_options)
        
        if self.options._list:
            response = self.client.get('/api/v1/buyer/appliances/list')
            appliances = self.parse_response(response)
            for appliance in appliances:
                print model.Appliance(appliance)
                                   
        elif self.options._key:
            response = self.client.get('/api/v1/buyer/appliance/%s' % self.options._key)
            appliance = self.parse_response(response)
            print model.Appliance(appliance)
                                 
        elif self.options._delete:
            response = self.client.delete('/api/v1/buyer/appliance/%s' % self.options._delete)
            self.parse_response(response, expected_status=httplib.NO_CONTENT)
            print 'Deleted\n'  
                     
        elif self.options._build:
            if not self.options._build.endswith('.img'):
                sys.exit('Invalid image type - expecting .img')
            if not os.path.exists(self.options._build):
                sys.exit('Invalid image path')
            print 'Building...'
            # TODO: unused "disk" variable 
            disk = ExistingImageDisk(DISK_TYPE_HD, self.options._build)
            print build_package(self.options._name, self.options._build)
                                        
        elif self.options._upload:
            print upload_package(self.client, self.options._upload)['edit_url']
            
        else:
            self.parser.print_help()
                    
    def usage(self):
        
        return 'spotsh appliance \n'
        
        
class ProviderCommand(BaseCommand):
    
    command_options = (
        optparse.make_option('-l',
                             '--list',
                             action='store_true',
                             dest='_list',
                             help='List all providers'),
                    
        optparse.make_option('-k',
                             '--key',
                             default=None,
                             dest='_key',
                             help='Retrieve provider by key'),
    )
    
    def __init__(self):
        
        super(ProviderCommand, self).__init__(self.command_options)
        
        if self.options._list:
            response = self.client.get('/api/v1/buyer/providers/list')
            providers = self.parse_response(response)
            for provider in providers:
                print model.Provider(provider)
            
        elif self.options._key:
            response = self.client.get('/api/v1/buyer/provider/%s' % self.options._key)
            provider = self.parse_response(response)
            print model.Provider(provider)
            
        else:
            self.parser.print_help()
                    
    def usage(self):
        return 'spotsh provider \n'


class HardwareCommand(BaseCommand):
    
    command_options = (
        optparse.make_option('-p',
                             '--provider',
                             default=None,
                             dest='_provider',
                             help='List hardware for the specified provider'),
                             
        optparse.make_option('-l',
                             '--list',
                             action='store_true',
                             default=None,
                             dest='_list',
                             help='List hardware profiles'),
        
        optparse.make_option('-u',
                             '--update',
                             default=None,
                             dest='_hardware',
                             help='Update hardware profile'),

        optparse.make_option('-c',
                             '--cost',
                             default=None,
                             dest='_cost',
                             help='Hardware profile cost'),

        optparse.make_option('-m',
                             '--max_hours',
                             default=None,
                             dest='_max_hours',
                             help='Hardware profile max_hours'),
    )
    
    def __init__(self):
        
        super(HardwareCommand, self).__init__(self.command_options)
            
        # buyer part 

        if self.options._provider:
            response = self.client.get('/api/v1/buyer/provider/%s' % 
                                                    self.options._provider)
            provider = self.parse_response(response)
            for hardware in provider['hardware']:
                print model.Hardware(hardware)
                
        # provider part 
        # TODO: separate
                
        elif self.options._list:
            response = self.client.get('/api/v1/provider/hardware/list')
            hardwares = self.parse_response(response)
            for hardware in hardwares:
                print model.Hardware(hardware)
            
        elif self.options._hardware:
            if not (self.options._cost and self.options._max_hours):
                print "No data for update"
                self.parser.print_help()
                return                
            
            data = {'cost': self.options._cost,
                        'max_hours': self.options._max_hours}
                        
            response = self.client.put('/api/v1/provider/hardware/%s' %
                                       self.options._hardware, data = data)
            self.check_response(response)
            print "Success"
        
        else:
            self.parser.print_help()
                    
    def usage(self):
        return 'spotsh hardware \n'


class LogoutCommand(BaseCommand):
    
    def __init__(self):
        if os.path.exists(ACCESS_TOKEN_PATH):
            os.remove(ACCESS_TOKEN_PATH)
            print "Success"
        else:
            print "Not logged"
    
def main():
    
    MainCommand()




#    spotsh - The SpotCloud command-line utility.
#    Copyright (C) 2011  Enomaly
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import os.path
import uuid
import httplib
import socket
from urllib import urlencode
from urlparse import urljoin

from oauth import oauth
from utils import to_str

OAUTH_GET_REQUEST_TOKEN_PATH    = '/_ah/OAuthGetRequestToken'
OAUTH_AUTHORIZE_TOKEN_PATH      = '/_ah/OAuthAuthorizeToken'
OAUTH_GET_ACCESS_TOKEN_PATH     = '/_ah/OAuthGetAccessToken'

URLENCODED_TYPE = 'application/x-www-form-urlencoded'

is_file = lambda o: hasattr(o, 'read')


def field_generator(boundary, name, value):
    yield '--' + boundary
    yield 'Content-Disposition: form-data; name="%s"' % to_str(name)
    yield ''
    yield to_str(value)

def file_generator(boundary, name, file):
    name = to_str(name)
    filename = to_str(os.path.basename(file.name))
    
    yield '--' + boundary
    yield 'Content-Disposition: form-data; name="%s"; filename="%s"' % (name, filename)
    yield 'Content-Type: application/octet-stream'
    yield ''
    yield file.read()

def multipart_generator(boundary, fields):
    for key, value in fields.items():
        if is_file(value):
            for chunk in file_generator(boundary, key, value):
                yield chunk
        else:
            for chunk in field_generator(boundary, key, value):
                yield chunk
    yield '--' + boundary + '--'
    yield ''

def encode_multipart(data):
    boundary = str(uuid.uuid4())
    body = '\r\n'.join(multipart_generator(boundary, data))
    headers = {
        'Content-Type': 'multipart/form-data; boundary=%s' % boundary,
        'Content-Length': len(body),
    }
    return headers, body


class ClientBase(object):
    
    def __init__(self, host):
        self.connection = httplib.HTTPSConnection(host)
    
    def sign_request(self, method, path, data, body, headers):
        pass
    
    def request(self, method, path, data=None, headers=None):
        body = None
        
        if headers:
            headers = headers.copy() 
        else:
            headers = {}
        
        if data:
            if method in ('GET', 'DELETE'):
                path += '?' + urlencode(data, doseq=True)
            elif method in ('POST', 'PUT'):
                content_headers, body = encode_multipart(data)
                headers.update(content_headers)
            else:
                raise ValueError('%s method is not supported' % method)
        
        self.sign_request(method, path, data, body, headers)
        
        try:
            self.connection.request(method, path, body, headers)
        except socket.gaierror:
            sys.exit('Host connection failed')
            
        response = self.connection.getresponse()
        
        return response
    
    def get(self, path, data=None, headers=None):
        return self.request('GET', path, data, headers)
    
    def post(self, path, data=None, headers=None):
        return self.request('POST', path, data, headers)
    
    def put(self, path, data=None, headers=None):
        return self.request('PUT', path, data, headers)
    
    def delete(self, path, data=None, headers=None):
        return self.request('DELETE', path, data, headers)


class SpotCloud(ClientBase):
    
    def __init__(self, host, consumer, access_token=None):
        
        super(SpotCloud, self).__init__(host)
        
        self.consumer = consumer
        self.request_token = None
        self.access_token = access_token
        self.host = host
    
    def fetch_oauth_request_token(self):
        response = self.get(OAUTH_GET_REQUEST_TOKEN_PATH)
        
        try:
            self.request_token = oauth.OAuthToken.from_string(response.read())
        except KeyError:
            return None
            
        return self.request_token
    
    def get_oauth_authorize_url(self):
        if self.request_token is None:
            raise RuntimeError('You should get request token first')
        
        url = urljoin('https://%s/' % self.host, OAUTH_AUTHORIZE_TOKEN_PATH)
        
        oauth_request = oauth.OAuthRequest.from_token_and_callback(
            token = self.request_token,
            http_url = url,
        )
        return oauth_request.to_url()
    
    def fetch_oauth_access_token(self):
        if self.request_token is None:
            raise RuntimeError('You should get request token first')
        
        response = self.get(OAUTH_GET_ACCESS_TOKEN_PATH)
        self.access_token = oauth.OAuthToken.from_string(response.read())
        return self.access_token
    
    def sign_request(self, method, path, data, body, headers):
        url = urljoin('https://%s/' % self.host, path)
        
        parameters = None
        if body is None or headers.get('Content-Type') == URLENCODED_TYPE:
            parameters = data
        
        signature_method = oauth.OAuthSignatureMethod_HMAC_SHA1()
        
        if self.access_token is not None:
            token = self.access_token
        elif self.request_token is not None:
            token = self.request_token
        else:
            token = None
        
        oauth_request = oauth.OAuthRequest.from_consumer_and_token(
            oauth_consumer = self.consumer,
            token = token,
            http_url = url,
            http_method = method,
            parameters = parameters,
        )
        oauth_request.sign_request(signature_method, self.consumer, token)
        headers.update(oauth_request.to_header())






#    spotsh - The SpotCloud command-line utility.
#    Copyright (C) 2011  Enomaly
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup, find_packages

setup(name="spotsh",
      version='0.0.2',
      # python 2.6+ already has json module
      install_requires = ['simplejson', 'oauth'],
      zip_safe=False,
      packages = find_packages('src'),
      package_dir = {'':'src'},
      author = "Enomaly",
      author_email = "adam@enomaly.com",
      license = "GPL V3",
      url = "http://www.enomaly.com/",
      entry_points = {'console_scripts': ['spotsh = spotsh:main']})
